Index: channels/chan_zap.c
===================================================================
RCS file: /usr/cvsroot/asterisk/channels/chan_zap.c,v
retrieving revision 1.397
diff -u -p -r1.397 chan_zap.c
--- channels/chan_zap.c	7 Jan 2005 21:18:00 -0000	1.397
+++ channels/chan_zap.c	9 Jan 2005 20:17:19 -0000
@@ -271,6 +271,9 @@ AST_MUTEX_DEFINE_STATIC(iflock);
 
 static int ifcount = 0;
 
+/* Whether we answer on a Polarity Switch event */
+static int answeronpolarityswitch = 0;
+
 /* Whether we hang up on a Polarity Switch event */
 static int hanguponpolarityswitch = 0;
 
@@ -402,6 +405,10 @@ struct zt_pri;
 #define SUB_CALLWAIT	1			/* Call-Waiting call on hold */
 #define SUB_THREEWAY	2			/* Three-way call */
 
+/* Polarity states */
+#define POLARITY_IDLE   0
+#define POLARITY_REV    1
+
 
 static struct zt_distRings drings;
 
@@ -436,6 +443,7 @@ struct zt_subchannel {
 	int needflash;
 	int linear;
 	int inthreeway;
+	int polarity;
 	ZT_CONFINFO curconf;
 };
 
@@ -566,6 +574,7 @@ static struct zt_pvt {
 	int fake_event;
 	int zaptrcallerid;	/* should we use the callerid from incoming call on zap transfer or not */
 	int emdigitwait;
+	int answeronpolarityswitch;
 	int hanguponpolarityswitch;
 	int polarityonanswerdelay;
 	struct timeval polaritydelaytv;
@@ -873,6 +882,7 @@ static int unalloc_sub(struct zt_pvt *p,
 	p->subs[x].chan = 0;
 	p->subs[x].owner = NULL;
 	p->subs[x].inthreeway = 0;
+	p->subs[x].polarity = POLARITY_IDLE;
 	memset(&p->subs[x].curconf, 0, sizeof(p->subs[x].curconf));
 	return 0;
 }
@@ -2014,6 +2024,7 @@ static int zt_hangup(struct ast_channel 
 		p->subs[index].needcongestion = 0;
 		p->subs[index].linear = 0;
 		p->subs[index].needcallerid = 0;
+		p->subs[index].polarity = POLARITY_IDLE;
 		zt_setlinear(p->subs[index].zfd, 0);
 		if (index == SUB_REAL) {
 			if ((p->subs[SUB_CALLWAIT].zfd > -1) && (p->subs[SUB_THREEWAY].zfd > -1)) {
@@ -3179,7 +3190,7 @@ static struct ast_frame *zt_handle_event
 							ast_log(LOG_DEBUG, "Done dialing, but waiting for progress detection before doing more...\n");
 						} else if (p->confirmanswer || (!p->dialednone && ((p->sig == SIG_EM) || (p->sig == SIG_EM_E1) ||  (p->sig == SIG_EMWINK) || (p->sig == SIG_FEATD) || (p->sig == SIG_FEATDMF) || (p->sig == SIG_E911) || (p->sig == SIG_FEATB) || (p->sig == SIG_SF) || (p->sig == SIG_SFWINK) || (p->sig == SIG_SF_FEATD) || (p->sig == SIG_SF_FEATDMF) || (p->sig == SIG_SF_FEATB)))) {
 							ast_setstate(ast, AST_STATE_RINGING);
-						} else {
+						} else if (!p->answeronpolarityswitch) {
 							ast_setstate(ast, AST_STATE_UP);
 							p->subs[index].f.frametype = AST_FRAME_CONTROL;
 							p->subs[index].f.subclass = AST_CONTROL_ANSWER;
@@ -3721,14 +3732,22 @@ static struct ast_frame *zt_handle_event
 		case ZT_EVENT_POLARITY:
 			/*
 			 * If we get a Polarity Switch event, check to see
-			 * if it should be ignored (the off-hook action seems
-			 * to cause a Polarity Switch) or whether it is an
-			 * indication of remote end disconnect, in which case
-			 * we should hang up
+			 * if we should change the polarity state and
+			 * mark the channel as UP or if this is an indication
+			 * of remote end disconnect. 
 			 */
-
-			if(p->hanguponpolarityswitch &&
+			if (p->subs[SUB_REAL].polarity == POLARITY_IDLE) {
+				p->subs[SUB_REAL].polarity = POLARITY_REV;
+				if (p->answeronpolarityswitch &&
+				    ((ast->_state == AST_STATE_DIALING) ||
+				     (ast->_state == AST_STATE_RINGING))) {
+					ast_log(LOG_DEBUG, "Answering on polarity switch!\n");
+					ast_setstate(p->owner, AST_STATE_UP);
+				} else 
+					ast_log(LOG_DEBUG, "Ignore switch to REVERSED Polarity on channel %d, state %d\n", p->channel, ast->_state);
+			} else if(p->hanguponpolarityswitch &&
 				(p->polarityonanswerdelay > 0) &&
+			        (p->subs[SUB_REAL].polarity == POLARITY_REV) &&
 				(ast->_state == AST_STATE_UP)) {
 				struct timeval tv;
 				gettimeofday(&tv, NULL);
@@ -3736,11 +3755,13 @@ static struct ast_frame *zt_handle_event
 				if((((tv.tv_sec - p->polaritydelaytv.tv_sec) * 1000) + ((tv.tv_usec - p->polaritydelaytv.tv_usec)/1000)) > p->polarityonanswerdelay) {
 					ast_log(LOG_DEBUG, "Hangup due to Reverse Polarity on channel %d\n", p->channel);
 					ast_softhangup(p->owner, AST_SOFTHANGUP_EXPLICIT);
+					p->subs[SUB_REAL].polarity = POLARITY_IDLE;
 				} else {
 					ast_log(LOG_DEBUG, "Ignore Reverse Polarity (too close to answer event) on channel %d, state %d\n", p->channel, ast->_state);
 				}
 			} else {
-				ast_log(LOG_DEBUG, "Ignore Reverse Polarity on channel %d, state %d\n", p->channel, ast->_state);
+				p->subs[SUB_REAL].polarity = POLARITY_IDLE;
+				ast_log(LOG_DEBUG, "Ignore switch to IDLE Polarity on channel %d, state %d\n", p->channel, ast->_state);
 			}
 			break;
 		default:
@@ -5839,6 +5860,7 @@ static int handle_init_event(struct zt_p
 		case SIG_FXSKS:
 		case SIG_FXSGS:
 			if (i->cid_start == CID_START_POLARITY) {
+				i->subs[SUB_REAL].polarity = POLARITY_REV;
 				ast_verbose(VERBOSE_PREFIX_2 "Starting post polarity "
 					    "CID detection on channel %d\n",
 					    i->channel);
@@ -6640,6 +6662,7 @@ static struct zt_pvt *mkintf(int channel
 		}
 
 		tmp->polarityonanswerdelay = polarityonanswerdelay;
+		tmp->answeronpolarityswitch = answeronpolarityswitch;
 		tmp->hanguponpolarityswitch = hanguponpolarityswitch;
 		tmp->sendcalleridafter = sendcalleridafter;
 
@@ -9798,6 +9821,8 @@ static int setup_zap(int reload)
 				cur_emdigitwait = atoi(v->value);
 			} else if (!strcasecmp(v->name, "polarityonanswerdelay")) {
 				polarityonanswerdelay = atoi(v->value);
+			} else if (!strcasecmp(v->name, "answeronpolarityswitch")) {
+				answeronpolarityswitch = ast_true(v->value);
 			} else if (!strcasecmp(v->name, "hanguponpolarityswitch")) {
 				hanguponpolarityswitch = ast_true(v->value);
 			} else if (!strcasecmp(v->name, "sendcalleridafter")) {
Index: configs/zapata.conf.sample
===================================================================
RCS file: /usr/cvsroot/asterisk/configs/zapata.conf.sample,v
retrieving revision 1.36
diff -u -p -r1.36 zapata.conf.sample
--- configs/zapata.conf.sample	28 Dec 2004 21:32:25 -0000	1.36
+++ configs/zapata.conf.sample	9 Jan 2005 20:17:19 -0000
@@ -325,11 +325,16 @@ immediate=no
 ;
 ;busycount=4
 ;
+; Use a polarity reversal to mark when a outgoing call is answered by the
+; remote party.
+;
+;answeronpolarityswitch=yes
+;
 ; In some countries, a polarity reversal is used to signal the disconnect
 ; of a phone line.  If the hanguponpolarityswitch option is selected, the
 ; call will be considered "hung up" on a polarity reversal
 ;
-;hanguponpolarityswitch
+;hanguponpolarityswitch=yes
 ;
 ; On trunk interfaces (FXS) it can be useful to attempt to follow the progress
 ; of a call through RINGING, BUSY, and ANSWERING.   If turned on, call
